@@ -23,13 +23,13 @@
 
 
 -------------------------------------------------------------------------------
-FILENAME:                utils.c
+FILENAME:                imgproc_deform.c
 LIBRARY:                 libgpiv:
 EXTERNAL FUNCTIONS:      gpiv_imgproc_deform
 
 LOCAL FUNCTIONS:         compare_float
 
-LAST MODIFICATION DATE:  $Id: imgproc_deform.c,v 1.2 2006/02/24 17:23:20 gerber Exp $
+LAST MODIFICATION DATE:  $Id: imgproc_deform.c,v 1.6 2007-12-19 08:46:46 gerber Exp $
 
 -------------------------------------------------------------------------------
 
@@ -63,89 +63,191 @@ LAST MODIFICATION DATE:  $Id: imgproc_deform.c,v 1.2 2006/02/24 17:23:20 gerber
  * Prototyping of static functions
  */
 static void
-ConvertToInterpolationCoefficients(
-				   double	c[],	        /* input samples --> output coefficients */
-                                   long	DataLength,	        /* number of samples or coefficients */
-                                   double	z[],		/* poles */
-                                   long	NbPoles,	        /* number of poles */
-                                   double	Tolerance	/* admissible relative error */
-                                   );
+ConvertToInterpolationCoefficients
+				(double	c[],	        /* input samples --> output coefficients */
+				long	DataLength,	/* number of samples or coefficients */
+				double	z[],		/* poles */
+				long	NbPoles,        /* number of poles */
+				double	Tolerance	/* admissible relative error */
+				);
 
 static void 
-GetColumn(
-	  float	*Image,		                               /* input image array */
-          long	Width,		                               /* width of the image */
-          long	x,			                       /* x coordinate of the selected line */
-          double	Line[],		                       /* output linear array */
-          long	Height		                               /* length of the line and height of the image */
-          );
+GetColumn			(float	*Image,		/* input image array */
+				long	Width,		/* width of the image */
+				long	x,		/* x coordinate of the selected line */
+				double	Line[],		/* output linear array */
+				long	Height		/* length of the line and height of the image */
+				);
 
 static void
-GetRow(
-       float	*Image,		/* input image array */
-       long	y,		/* y coordinate of the selected line */
-       double	Line[],		/* output linear array */
-       long	Width		/* length of the line and width of the image */
-       );
+GetRow				(float	*Image,		/* input image array */
+				long	y,		/* y coordinate of the selected line */
+				double	Line[],		/* output linear array */
+				long	Width		/* length of the line and width of the image */
+				);
 
 static double
-InitialCausalCoefficient(
-			 double	c[],		/* coefficients */
-			 long	DataLength,	/* number of coefficients */
-			 double	z,			/* actual pole */
-			 double	Tolerance	/* admissible relative error */
-			 );
+InitialCausalCoefficient	(double	c[],		/* coefficients */
+				long	DataLength,	/* number of coefficients */
+				double	z,	       	/* actual pole */
+				double	Tolerance	/* admissible relative error */
+				);
 
 static double
-InitialAntiCausalCoefficient(
-			     double	c[],		/* coefficients */
-			     long	DataLength,	/* number of samples or coefficients */
-			     double	z			/* actual pole */
-			     );
+InitialAntiCausalCoefficient	(double	c[],		/* coefficients */
+				long	DataLength,	/* number of samples or coefficients */
+				double	z	       	/* actual pole */
+				);
 
 static void
-PutColumn(
-	  float	*Image,		/* output image array */
-	  long	Width,		/* width of the image */
-	  long	x,			/* x coordinate of the selected line */
-	  double	Line[],		/* input linear array */
-	  long	Height		/* length of the line and height of the image */
-	  );
+PutColumn			(float	*Image,		/* output image array */
+				long	Width,		/* width of the image */
+				long	x,		/* x coordinate of the selected line */
+				double	Line[],		/* input linear array */
+				long	Height		/* length of the line and height of the image */
+				);
 
 static void
-PutRow (
-	float	*Image,		/* output image array */
-	long	y,			/* y coordinate of the selected line */
-	double	Line[],		/* input linear array */
-	long	Width		/* length of the line and width of the image */
-	);
+PutRow				(float	*Image,		/* output image array */
+				long	y,		/* y coordinate of the selected line */
+				double	Line[],		/* input linear array */
+				long	Width		/* length of the line and width of the image */
+				);
 
 
 
 static int
-SamplesToCoefficients (
-		       float	*Image,		/* in-place processing */
-		       long	Width,		/* width of the image */
-		       long	Height,		/* height of the image */
-		       long	SplineDegree    /* degree of the spline model */
-		       );
+SamplesToCoefficients		(float	*Image,		/* in-place processing */
+				long	Width,		/* width of the image */
+				long	Height,		/* height of the image */
+				long	SplineDegree    /* degree of the spline model */
+				);
 
 static int
-img_deform(guint16 **img_src,
-           guint16 **img_dest, 
-           GpivImagePar *image_par,
-           GpivPivData piv_data
-           );
+img_deform_frame		(guint16	**img,
+				GpivImagePar	*image_par,
+				GpivPivData	*piv_data
+				);
 
 static double	
-InterpolatedValue(
-                  float	*Bcoeff,	/* input B-spline array of coefficients */
-		  long   Width,		/* width of the image */
-		  long   Height,	/* height of the image */
-		  double x,	      	/* x coordinate where to interpolate */
-		  double y,	       	/* y coordinate where to interpolate */
-		  long SplineDegree     /* degree of the spline model */
-		  );
+InterpolatedValue		(float	*Bcoeff,	/* input B-spline array of coefficients */
+				long	Width,		/* width of the image */
+				long	Height,		/* height of the image */
+				double	x,	      	/* x coordinate where to interpolate */
+				double	y,	       	/* y coordinate where to interpolate */
+				long	SplineDegree	/* degree of the spline model */
+				);
+
+/*
+ * Public functions
+ */
+gchar *
+gpiv_imgproc_deform (GpivImage *image,
+                     const GpivPivData *piv_data
+                     )
+/*-----------------------------------------------------------------------------
+ * DESCRIPTION:
+ *     Image shifting and deformation routine for a single exposed, double 
+ *     frame PIV image pair with magnitude of PIV estimations at each pixel
+ *     Deforms first frame halfway forward and second frame halfway backward.
+ *
+ *---------------------------------------------------------------------------*/
+{
+    gchar *err_msg = NULL;
+    gint i, j;
+    GpivPivData *pd = NULL;
+
+    gint k, l, count = 0, window = 4;
+    gfloat dy_sum, dx_sum;
+
+
+    if (image->header->x_corr__set == FALSE
+        || image->header->x_corr == FALSE) {
+        err_msg = "gpiv_imgproc_deform: only to be used for cross correlation";
+        return (err_msg);
+    }
+
+    if (image->frame1[0] == NULL) {
+        err_msg = "gpiv_imgproc_deform: image->frame1 == NULL";
+        return (err_msg);
+    }
+
+    if (image->frame2[0] == NULL) {
+        err_msg = "gpiv_imgproc_deform: image->frame2 == NULL";
+        return (err_msg);
+    }
+
+    /*
+     * Interpolate piv data at each pixel point of the images.
+     */
+    pd = gpiv_alloc_pivdata (image->header->ncolumns, 
+                             image->header->nrows);
+    for (i = 0; i < pd->ny; i++) {
+        for (j = 0; j < pd->nx; j++) {
+            pd->point_x[i][j] = (float) j;
+            pd->point_y[i][j] = (float) i;
+        }
+    }
+    gpiv_piv_dxdy_at_new_grid (piv_data, pd);
+
+    /*
+     * Deform first image frame with half shift forwards.
+     */
+/*     pd2 = gpiv_cp_pivdata (pd); */
+
+#undef SMOOTH
+#ifdef SMOOTH
+    /*
+     * Smoothing
+     */
+    for (i = 0; i < pd->ny; i++) {
+        for (j = 0; j < pd->nx; j++) {
+            for (k = -window / 2; k < window / 2; k++) {
+                if (i + k > 0 && i + k < pd->ny) {
+                    for (l = -window / 2; l < window / 2; l++) {
+                        if (j + l > 0 && j + l < pd->nx) {
+                            dy_sum += pd->dy[i+k][j+l];
+                            dx_sum += pd->dx[i+k][j+l];
+                            count++;
+                        }
+                    }
+                }       
+            }
+            count = 0;
+        }
+    }
+#endif /* SMOOTH */
+
+    for (i = 0; i < pd->ny; i++) {
+        for (j = 0; j < pd->nx; j++) {
+            pd->dx[i][j] = 0.50 * pd->dx[i][j];
+            pd->dy[i][j] = 0.50 * pd->dy[i][j];
+        }
+    }
+
+    img_deform_frame (image->frame1, image->header, pd);
+
+    /*
+     * Deform second image frame with half shift backwards.
+     */
+    for (i = 0; i < pd->ny; i++) {
+        for (j = 0; j < pd->nx; j++) {
+            pd->dx[i][j] = -1.00 * pd->dx[i][j];
+            pd->dy[i][j] = -1.00 * pd->dy[i][j];
+        }
+    }
+
+    img_deform_frame (image->frame2, image->header, pd);
+
+
+    gpiv_free_pivdata (pd);
+    return (err_msg);
+#ifdef SMOOTH
+#undefine SMOOTH
+#endif
+}
+
+
 
 /*
  * Definition of static functions
@@ -153,7 +255,7 @@ InterpolatedValue(
 
 static void
 ConvertToInterpolationCoefficients (
-				    double	c[],		/* input samples --> output coefficients */
+                                    double	c[],		/* input samples --> output coefficients */
 				    long	DataLength,	/* number of samples or coefficients */
 				    double	z[],		/* poles */
 				    long	NbPoles,	/* number of poles */
@@ -197,7 +299,7 @@ ConvertToInterpolationCoefficients (
 static double
 InitialCausalCoefficient (
 			  double	c[],	/* coefficients */
-			  long	DataLength,	/* number of coefficients */
+			  long	        DataLength,	/* number of coefficients */
 			  double	z,	/* actual pole */
 			  double	Tolerance	/* admissible relative error */
 			  )
@@ -241,10 +343,10 @@ InitialCausalCoefficient (
 static void
 GetColumn (
 	   float	*Image,		/* input image array */
-	   long	Width,		        /* width of the image */
-	   long	x,			/* x coordinate of the selected line */
+	   long	        Width,		        /* width of the image */
+	   long	        x,			/* x coordinate of the selected line */
 	   double	Line[],		/* output linear array */
-	   long	Height		        /* length of the line */
+	   long	        Height		        /* length of the line */
 	   )
 /*--------------------------------------------------------------------------*/
 { /* begin GetColumn */
@@ -609,10 +711,9 @@ InterpolatedValue(
 
 
 static int
-img_deform(guint16 **img_src,
-           guint16 **img_dest, 
-           GpivImagePar *image_par,
-           GpivPivData piv_data
+img_deform_frame (guint16 **img,
+                  GpivImagePar *image_par,
+                  GpivPivData *piv_data
            )
 /*-----------------------------------------------------------------------------
  * DESCRIPTION:
@@ -622,16 +723,16 @@ img_deform(guint16 **img_src,
  * INPUTS:
  *      piv_data:       PIV data structure which defines shift/deformation at 
  *                      each image pixel
- *      img_src:        input image frame
+ *      img:            input image frame
  *
  * OUTPUTS:
- *      img_dest:       deformed image frame
+ *      img:            deformed image frame
  *
  * RETURNS:
  *
  *---------------------------------------------------------------------------*/
 {
-    float	*ImageRasterArray, *OutputImage;
+    float	*ImageRasterArray = NULL, *OutputImage = NULL;
     float	*p, *q;
     double	x1, y1;
     long	Width = image_par->ncolumns, Height = image_par->nrows;
@@ -640,6 +741,7 @@ img_deform(guint16 **img_src,
     int		Masking = 1;
     int		Error;
 
+    guint img_top = (1 << image_par->depth) - 1;
     unsigned char *Line;
     gint i, j;
 
@@ -651,11 +753,11 @@ img_deform(guint16 **img_src,
     long	rounded;
 
 
-    ImageRasterArray = gpiv_vector(image_par->nrows * image_par->ncolumns);
+    ImageRasterArray = gpiv_vector (image_par->nrows * image_par->ncolumns);
     p = ImageRasterArray;
     for (i = 0; i < image_par->nrows; i++) {
         for (j = 0; j < image_par->ncolumns; j++) {
-            *p++ = (float) img_src[i][j];
+            *p++ = (float) img[i][j];
         }
     }
 
@@ -664,182 +766,62 @@ img_deform(guint16 **img_src,
  * convert between a representation based on image samples
  * and a representation based on image B-spline coefficients
  */
-    if (Error = SamplesToCoefficients(ImageRasterArray, Width, Height, 
-                                      SplineDegree)) {
-        gpiv_free_vector(ImageRasterArray);
-        g_warning("img_deform: Change of basis failed");
+    if (Error = SamplesToCoefficients (ImageRasterArray, Width, Height, 
+                                       SplineDegree)) {
+        gpiv_free_vector (ImageRasterArray);
+        g_warning("LIBGPIV internal error: img_deform: Change of basis failed");
         return(1);
     }
 
 /*
  * visit all pixels of the output image and assign their value
  */
-    OutputImage = gpiv_vector(image_par->nrows * image_par->ncolumns);
+    OutputImage = gpiv_vector (image_par->nrows * image_par->ncolumns);
     p = OutputImage;
     for (i = 0; i < image_par->nrows; i++) {
         for (j = 0; j < image_par->ncolumns; j++) {
-            x1 = (double) j - (double) piv_data.dx[i][j];
-            y1 = (double) i - (double) piv_data.dy[i][j];
+            x1 = (double) j - (double) piv_data->dx[i][j];
+            y1 = (double) i - (double) piv_data->dy[i][j];
             if (Masking) {
                 if ((x1 <= -0.5) 
                     || (((double)Width - 0.5) <= x1)
                     || (y1 <= -0.5) 
                     || (((double)Height - 0.5) <= y1)) {
                     *p++ = 0.0F;
+                } else {
+                    *p++ = (float)InterpolatedValue (ImageRasterArray, 
+                                                     Width, Height,
+                                                     x1, y1, SplineDegree);
                 }
-                else {
-                    *p++ = (float)InterpolatedValue
-                        (ImageRasterArray, Width, Height,
-                         x1, y1, SplineDegree);
-                }
-            }
-            else {
-                *p++ = (float)InterpolatedValue
-                    (ImageRasterArray, Width, Height,
-                     x1, y1, SplineDegree);
+            } else {
+                *p++ = (float)InterpolatedValue (ImageRasterArray, 
+                                                 Width, Height,
+                                                 x1, y1, SplineDegree);
             }
 
         }
     }
 
 /*
- * Store back into img_dest
+ * Store back into img
  */
 
     p = OutputImage;
     for (i = 0; i < image_par->nrows; i++) {
         for (j = 0; j < image_par->ncolumns; j++) {
-            rounded = (long)floor((double)*p++ + 0.5);
+            rounded = (long)floor ((double)*p++ + 0.5);
             if (rounded < 0.0) rounded = 0.0;
-            if (rounded > 255.0) rounded = 255.0;
-            img_dest[i][j] =  (guint16) rounded;
+            if (rounded > (gfloat) img_top /* 255.0 */) rounded = (gfloat) img_top /* 255.0 */;
+            img[i][j] =  (guint16) rounded;
         }
     }
 
 
 
-    gpiv_free_vector(ImageRasterArray);
-    gpiv_free_vector(OutputImage);
+    gpiv_free_vector (ImageRasterArray);
+    gpiv_free_vector (OutputImage);
     return(0);
 }
 
 
 
-/*
- * Public functions
- */
-void 
-gpiv_imgproc_deform(GpivPivData * piv_data,
-                    GpivImagePar *image_par,
-                    guint16 **img1_src,
-                    guint16 **img2_src,
-                    guint16 **img1_dest,
-                    guint16 **img2_dest
-                    )
-/*-----------------------------------------------------------------------------
- * DESCRIPTION:
- *      Image shifting and deformation routine for a single exposed, double 
- *      frame PIV image pair with magnitude of PIV estimations at each pixel
- *
- * INPUTS:
- *      piv_data:       Input PIV data structure
- *      image_par:      image parameters
- *      img1_src:       first input image frame
- *      img2_src:       second input image frame
- *
- * OUTPUTS:
- *      img1_dest:      first deformed image frame
- *      img2_dest:      second deformed image frame
- *
- * RETURNS:
- *
- *---------------------------------------------------------------------------*/
-{
-    gint i, j;
-    GpivPivData pd, pd2;
-
-    gint k, l, count = 0, window = 4;
-    float dy_sum, dx_sum;
-
-/*     g_message("gpiv_imgproc_deform:: 1 "); */
-/*
- * Interpolate piv data at each pixel point of the images.
- */
-    pd.nx = image_par->ncolumns;
-    pd.ny = image_par->nrows;
-    gpiv_null_pivdata(&pd);
-    gpiv_alloc_pivdata(&pd);
-    for (i = 0; i < pd.ny; i++) {
-        for (j = 0; j < pd.nx; j++) {
-            pd.point_x[i][j] = (float) j;
-            pd.point_y[i][j] = (float) i;
-        }
-    }
-
-
-    gpiv_piv_dxdy_at_new_grid(*piv_data, &pd);
-/*     g_message("gpiv_imgproc_deform:: gpiv_write_pivdata with piv_data"); */
-/*     gpiv_write_pivdata(piv_data, NULL, 0, 0, NULL); */
-/*     g_message("gpiv_imgproc_deform:: gpiv_write_pivdata with pd"); */
-/*     gpiv_write_pivdata(&pd, NULL, 0, 0, NULL); */
-
-    pd2.nx = pd.nx;
-    pd2.ny = pd.ny;
-    gpiv_null_pivdata(&pd2);
-    gpiv_alloc_pivdata(&pd2);
-/*
- * Deform first image frame with half shift forwards.
- */
-    gpiv_cp_pivdata(&pd, &pd2);
-/*
- * Smoothing
- */
-/*     for (i = 0; i < pd.ny; i++) { */
-/*         for (j = 0; j < pd.nx; j++) { */
-/*             for (k = -window / 2; k < window / 2; k++) { */
-/*                 if (i + k > 0 && i + k < pd.ny) { */
-/*                     for (l = -window / 2; l < window / 2; l++) { */
-/*                         if (j + l > 0 && j + l < pd.nx) { */
-/*                             dy_sum += pd.dy[i+k][j+l]; */
-/*                             dx_sum += pd.dx[i+k][j+l]; */
-/*                             count++; */
-/*                         } */
-/*                     } */
-/*                 }        */
-/*             } */
-/*             g_message(":: i = %d j = %d count = %d", i, j, count); */
-/*             pd2.dy[i][j] = dy_sum / (float) count; */
-/*             pd2.dx[i][j] = dx_sum / (float) count; */
-/*             count = 0; */
-/*         } */
-/*     } */
-/*     gpiv_fwrite_pivdata ("/home/gerber/Piv/Demo/tst.piv", &pd2, NULL, 0, 0,  */
-/*                          NULL); */
-
-    for (i = 0; i < pd.ny; i++) {
-        for (j = 0; j < pd.nx; j++) {
-            pd2.dx[i][j] = 0.50 * pd.dx[i][j];
-            pd2.dy[i][j] = 0.50 * pd.dy[i][j];
-        }
-    }
-    img_deform(img1_src, img1_dest, image_par, pd2);
-
-/*
- * Deform second image frame with half shift backwards.
- */
-    gpiv_cp_pivdata(&pd, &pd2);
-    for (i = 0; i < pd.ny; i++) {
-        for (j = 0; j < pd.nx; j++) {
-            pd2.dx[i][j] = -0.50 * pd.dx[i][j];
-            pd2.dy[i][j] = -0.50 * pd.dy[i][j];
-        }
-    }
-    img_deform(img2_src, img2_dest, image_par, pd2);
-
-
-    gpiv_free_pivdata(&pd);
-    gpiv_free_pivdata(&pd2);
-}
-
-
-
